using System;
using System.Data;
using System.IO;

/// History:
/// 
/// Initial Release: 05.10.2004
/// 
///	09.10.2004	-	added can read the original epg.data File; was not Unix-Textfile aware before
///				-	fixed crashed when no Event Title
///					(Daniel Kirstenpfad)
///	17.10.2004	-	fixed parse_EPG() Encoding Bug; now epg.data is parsed as ANSI not as UTF-8
///					(Matthias Eller)
///	17.10.2004	-	added Helper Routines for Unix<>DateTime Conversion
///	18.10.2004	-	added DateTime Conversion to parse_EPG()
///	
///	
namespace VDRGUI
{
	/// <summary>
	/// liest Daten aus epg.data Datei
	/// </summary>
	public class ParserAndData
	{
		#region interne Objekte
		/// <summary>
		/// Pfad und Dateiname der epg.data Datei
		/// </summary>
		private static string pv_s_epgdata_filename;
		/// <summary>
		/// interner Container fr alle EPG Daten
		/// </summary>
		private static EPGData geparsteDaten = new EPGData();
		#endregion

		#region Hilfsmethoden
		/// <summary>
		/// Konvertiert einen Unix-Timestamp in .NET DateTime Informationen
		/// </summary>
		/// <param name="unixtimestamp"></param>
		/// <returns></returns>
		private DateTime UnixTimeStamp2DateTime(double unixtimestamp) 
		{
			DateTime temp = new DateTime(1970,1,1,0,0,0).AddSeconds(unixtimestamp);

			return(temp);
		}

		/// <summary>
		/// Konvertiert eine .NET DateTime Information in einen Unix-Timestamp
		/// </summary>
		/// <param name="temp_datetime">.NET DateTime Information</param>
		/// <returns>Unixtimestamp in Double</returns>
		private double DateTime2UnixTimeStamp(DateTime temp_datetime)
		{
			TimeSpan ts = (temp_datetime.ToUniversalTime() - new DateTime(1970,1,1,0,0,0));
			double unixTime = ts.TotalSeconds;
			
			return unixTime;
		}
		#endregion

		#region Konstruktor
		public ParserAndData(string s_epg_data_filename)
		{
			pv_s_epgdata_filename = s_epg_data_filename;
		}
		#endregion

		#region Parser
		/// <summary>
		/// parst das EPG und erzeugt die entsprechenden Datenstrukturen im Speicher
		/// liefert Statuscode zurck:
		///	0 = kein Fehler
		///	1 = epg.data nicht gefunden
		/// </summary>
		public int parse_EPG()
		{			
			if (!File.Exists(pv_s_epgdata_filename))
			{ 
				return 1; 
			}    
			//using (TextReader epgStreamReader = File.OpenText(pv_s_epgdata_filename))		// UTF-8
			using (StreamReader epgStreamReader = new StreamReader(pv_s_epgdata_filename,System.Text.Encoding.Default))		// bugfix 17.10.04 Matthias Eller
			{
				EPGData.channelRow aktuellerSender = null;
				EPGData.epgDataRow aktuellerEvent = null;

				while (epgStreamReader.Peek() >= 0) 
				{
					string temporary = null;
					string[] t_gesplittet;

					temporary = epgStreamReader.ReadLine();

					#region parsen
					switch(temporary[0])
					{
						case 'C':
							aktuellerSender = geparsteDaten.channel.NewchannelRow();
							temporary = temporary.Remove(0,2);
							t_gesplittet = temporary.Split(' ');
							aktuellerSender.channel_string = t_gesplittet[0];
							aktuellerSender.channel_name = t_gesplittet[1];
							geparsteDaten.channel.AddchannelRow(aktuellerSender);
							break;
						case 'c':
							//geparsteDaten.channel.AddchannelRow(aktuellerSender);
							break;
						case 'E':
							temporary = temporary.Remove(0,2);
							aktuellerEvent = geparsteDaten.epgData.NewepgDataRow();
							aktuellerEvent.channel_string = aktuellerSender.channel_string;
							aktuellerEvent.event_string = temporary;
							t_gesplittet = temporary.Split(' ');
							aktuellerEvent.event_start = UnixTimeStamp2DateTime(Convert.ToInt64(t_gesplittet[1]));
							aktuellerEvent.event_duration = Convert.ToInt64(t_gesplittet[2]);
							aktuellerEvent.event_title = "n/a";
							break;
						case 'T':
							temporary = temporary.Remove(0,2);
							aktuellerEvent.event_title = temporary;
							break;
						case 'D':
							temporary = temporary.Remove(0,2);
							aktuellerEvent.event_long_description = temporary;
							break;
						case 'S':
							temporary = temporary.Remove(0,2);
							aktuellerEvent.event_short_description = temporary;
							break;
						case 'V':
							temporary = temporary.Remove(0,2);
							aktuellerEvent.event_vps = Convert.ToInt64(temporary,10);
							break;
						case 'e':
							geparsteDaten.epgData.AddepgDataRow(aktuellerEvent);
							break;
					}
					#endregion
				}
			}
			return 0;
		}

		/// <summary>
		/// Parst die Ausgabe von LSTR (Liste der Aufnahmen) in die EPGData.recordings Tabelle
		/// </summary>
		/// <param name="returned_LSTR">Enthlt die Antwort auf das Kommando LSTR des VDR Servers</param>
		/// <returns>EPGData.recordings Tabelle</returns>
		public int parse_LSTR(string returned_LSTR)
		{
			/*
			 *	250-1 15.10 18:50* Die Simpsons
				.
				.
				.
				250 20 18.10 03:16* Twin Peaks~Schachmatt
			*/
			using (StringReader recordingsReader = new StringReader(returned_LSTR))
			{
				EPGData.recordingsRow aktuelleAufnahme = null;

				while (recordingsReader.Peek() >= 0) 
				{
					string temporary = null;
					string[] t_gesplittet;
					string[] t2_gesplittet;
					string[] t3_gesplittet;
					
					// Instanz erzeugen
					aktuelleAufnahme = geparsteDaten.recordings.NewrecordingsRow();

					// einlesen
					temporary = recordingsReader.ReadLine();
					
					// checken
					if (temporary.Length > 10)
					{
						// Start lschen
						temporary = temporary.Remove(0,4);
						// aufsplitten
						t_gesplittet = temporary.Split(' ');
						// 0 = Timer Nummer
						// 1 = Datum
						// 2 = Uhrzeit+ "*" fr noch nicht gesehen
						// 3 = Beschreibung der Aufnahme
						
						// check ob genug Elemente
						if (t_gesplittet.Length < 3)
						{
							throw new Exception("parse_LSTR Fehler beim parsen der LSTR() Ausgabe; Zeile zu kurz");
						}
						aktuelleAufnahme.recording_identifier = Convert.ToInt64(t_gesplittet[0],10);
						
						// Name der Aufnahme -> alles ab dem dritten Element
						
						string name = "";
						
						for (int i=3;i<t_gesplittet.Length;i++)
						{
							name = name+t_gesplittet[i]+" ";
						}
						
						name = name.Remove(name.Length-1,1);
						aktuelleAufnahme.recording_name = name;

						// Uhrzeit erzeugen
						// leider wird das Jahr nicht bermittelt, sodass immer das aktuelle Jahr angenommen
						// werden muss
						
						t2_gesplittet = t_gesplittet[1].Split('.');

						// schon gesehen Markierung auswerten und entfernen
						if (t_gesplittet[2].EndsWith("*"))
						{
							aktuelleAufnahme.seen = false;
							t_gesplittet[2] = t_gesplittet[2].Remove(t_gesplittet[2].Length-1,1);
						}
						else
							aktuelleAufnahme.seen = true;
						
						t3_gesplittet = t_gesplittet[2].Split(':');

						// endgltige Struktur erzeugen
						aktuelleAufnahme.recording_time = new DateTime(DateTime.Now.Year,Convert.ToInt32(t2_gesplittet[1]),Convert.ToInt32(t2_gesplittet[0]),Convert.ToInt32(t3_gesplittet[0]),Convert.ToInt32(t3_gesplittet[1]),0,0);
						
						// an Tabelle anfgen
						geparsteDaten.recordings.AddrecordingsRow(aktuelleAufnahme);
					}
					else
					{
						// Fehler
						throw new Exception("parse_LSTR Fehler beim parsen der LSTR() Ausgabe");
					}
				}
			}

			return 0;
		}

		#endregion

		#region Weiterverarbeitung
		/// <summary>
		/// erzeugt aus den geparsten EPG Daten XML Daten
		/// </summary>
		public string getXMLEPG()
		{
			return geparsteDaten.GetXml();
		}

		/// <summary>
		/// DEMO Methode die zeigt wie man auf die Daten zugreift
		/// </summary>
		public void DEMO_OutputData()
		{
			foreach (DataRow dr in geparsteDaten.channel.Rows)
			{
				Console.WriteLine("Sender: "+dr["channel_name"].ToString());
				foreach (DataRow dr2 in dr.GetChildRows("channelepgData"))
				{
					DateTime tempDate = (DateTime)dr2["event_start"];

					Console.WriteLine("\t"+tempDate.ToString());

					Console.WriteLine("\t\t" + dr2["event_title"].ToString());
				}
			}
		}
		#endregion

		#region Interface nach aussen
		/// <summary>
		/// Liefert die internen Daten nach draussen
		/// </summary>
		/// <returns>EPGData Struktur</returns>
		public EPGData GetherData()
		{
			return geparsteDaten;
		}
		/// <summary>
		/// Legt Daten in interne Daten ab
		/// </summary>
		/// <param name="Daten">EPGData Struktur von "aussen"</param>
		public void SetherData(EPGData Daten)
		{
			geparsteDaten = Daten;
		}
		#endregion
	}
}
